'''
|----------------------------------------------------------
|Sep 20, 2021 start  | Daniel T
|----------------------------------------------------------
|class: serial_esc2
|constant: ENQ=0x05,ACK=0X06
|method:
|   1. Ping , for connect
|   2. readData, read data
|   3. writeData, 
|------------------------------------------------------------ 
|Nov 26, 2021 add RS485
|method:
|   1.readData485
|   2.writeData485
|------------------------------------------------------------
|Dec 12, 2023 make Comport and USB port available for RS232
|------------------------------------------------------------
| June 19, 2024 add check ENQ command in readData
|------------------------------------------------------------
| March 7, 2025, adjust for Carousel

'''
#!/usr/bin/env python
import os
from signal import pause
import time
import serial
from serial.serialutil import Timeout
import logging
from esc2Kernel import rs485set                

ENQ =b'\x05'             #request for connect
ACK =b'\x06'              #respond
mStart = b'0x02'        #message start
mEnd = b'0x03'          #message end
mDelimiter = b'0x1f'    #message delimiter

#logging format
'''    
#file_log = open('Esc2_data.log','w').close() #close log file
format1 = '%(levelname)s--%(asctime)s--%(message)s'
logging.basicConfig(filename='/home/pi/esc2/log_file/Esc2_data_serial.log',
                    filemode='a',
                    format = format1,
                    level = logging.INFO)
                    #datefmt = '%H:%M:%S')
'''
class serialEsc():
    def __init__(self,rsType):
        if rsType=='rs232':
            rs_channel=os.popen('ls /dev/ttyUSB*').read()
            if 'ttyUSB0' in rs_channel:
                #using Mini uart port: ttyS0
                self.ser = serial.Serial(port='/dev/ttyUSB0',
                                    baudrate=9600,
                                    bytesize=serial.SEVENBITS,
                                    stopbits=serial.STOPBITS_ONE,
                                    parity=serial.PARITY_ODD,
                                    timeout=0
                                    )
            else:    
                #use PL011 UART port:ttyAMA0
                self.ser=serial.Serial(port='/dev/ttyAMA0',
                                        baudrate=9600,
                                        bytesize=7,
                                        parity=serial.PARITY_ODD,
                                        stopbits=1,
                                        timeout=0)
            
        else:
            #rs485rtu
            self.ser=serial.Serial(port='/dev/ttyAMA0',
                                    baudrate=19200,
                                    bytesize=8,
                                    parity=serial.PARITY_NONE,
                                    stopbits=1,
                                    timeout=0)
        self.connet_Flag=False
        self.Flag=0 #0 start to find 0x02 & 0x03
        self.ser.reset_input_buffer()    
        self.ser.reset_output_buffer()
        self.ds=b'\0x02'
        self.No_x03=0
        

    def __def__(self):#destructor
        self.closePort()
        
    def Ping(self):
        dataPing=self.ser.read()  
        if dataPing==ENQ:
            self.ser.write(ACK)
        elif dataPing==ACK:
            return True
        else :         
            return False
        
    def sendData(self,strD):
        i=0
        for i in range(len(strD)): 
            t=strD[i].encode('ascii','ignore')  
            self.ser.write(t)
            #time.sleep(0.025) # comunication for window xp
        
    def readData(self):
        #time.sleep(0.1)
        bufferSize=self.ser.in_waiting 
       
        if bufferSize :
            data =self.ser.read(bufferSize)
            #Convert bytes to string
            #string_sequence=data.decode('utf-8',error='ignore')
            #how remove
            if data==ENQ:
                self.ser.write(ACK)
                return False
            else:  
                print(f'data= {data} \n')
                #logging.info(data)
                if self.Flag==0:
                    t0=data.find(b'\x02')
                    if t0>=0 :
                        t2=data.find(b'\x03')
                        if t2==-1:
                            self.Flag=1 #0x03 not find
                            self.ds=data[t0:]
                            return False
                        else :
                            self.No_x03 +=1 #0x03 find
                            t1=data.find(b'\x02',t2+1)
                            if t1 ==-1: #complete sting found 10x03
                                self.No_x03=0
                                return data[t0:]
                            else:
                                t2=data.find(b'\x03',t1+1)
                                if t2==-1:
                                    self.ds=data[t0:]
                                    self.Flag=1
                                    return False
                                else: 
                                    self.No_x03 +=1 #find second 0x03
                                    t1=data.find(b'\x03',t2+1)
                                    if t1==-1:
                                        #total 0x03 is 2 in string
                                        t2=data.find(b'\x02',t1+1)
                                        if t2 ==-1:
                                            #only two 0x02
                                            self.fs.ds=data[t0:]
                                            new_data=self.getRecycle()
                                            self.No_x03=0
                                            return new_data
                                        else:
                                            self.ds=data[t0:]
                                            self.Flag=1
                                            return False
                                    else:
                                        #third 0x03 find
                                        self.ds=data[t0:]
                                        new_data=self.getData()
                                        self.Flag=0
                                        self.No_x03=0
                                        return new_data                                                   
                else :
                    t1=data.find(b'\x03')
                    self.ds +=data
                    if t1==-1:
                        return False
                    else:
                        if self.No_x03==2:# self.ds has 2 0x03
                            self.No_x03 +=1  #data has 1 . total has 3 0x03
                            new_data=self.getData()
                            self.Flag=0
                            self.No_x03=0
                            return new_data
                        elif self.No_x03==1: #fs.ds has 1 0x03
                            self.No_x03 +=1 #new data get 1, string Has 2
                            t2=data.find(b'\x03',t1+1)
                            if t2 !=-1: #string has 2 0x03
                                t2=data.find(b'\x02',t1) #if has extra data
                                if t2 !=-1:
                                    new_data=self.getRecycle() #no extra data
                                    self.Flag=0
                                    self.No_x03=0
                                    return new_data
                                else:    
                                     return False
                            else:               #string has 3
                                new_data=self.getData()
                                self.Flag=0
                                self.No_x03=0
                                return new_data
                        else: #ds no 0x03
                            self.No_x03 +=1 #data has 1 x03
                            t2=data.find(b'\x02',t1+1)
                            if t2 ==-1:
                                self.Flag=0
                                self.No_x03=0
                                return self.ds
                            else:
                                t1=data.find(b'\x03',t2+1)
                                if t1 ==-1:
                                    return False
                                else:
                                    self.No_x03 +=1 #data has 2 x03
                                    t2=data.find(b'\x02',t1+1)
                                    if t2 != -1:
                                        self.Flag=0
                                        self.No_x03=0
                                        new_data=self.getRecycle()
                                        return new_data
                                    else:
                                        t1=data.find(b'\x03',t2+1)
                                        if t1 !=-1:
                                            self.Flag=0
                                            self.No_x03=0
                                            new_data=self.getData()
                                            return new_data
                                        else:    
                                            return False
                                        
            return False
    
    def readData485(self):
        
        bufferSize=self.ser.in_waiting  
        if bufferSize :
            data =self.ser.read(bufferSize)
            return data
        else:
            data=[0]
            return data
    #CRC check error
    def getData(self):
        No1=3
        No2_1=self.ds.find(b'\x1f',No1)
        len=No2_1-No1
        temStr=bytearray(len)
        for i in range(len):    
            temStr[i]=data[No1+i] 

        #extract first batch Number           
        No1=No2_1+1
        No2=data.find(b'\x1f',No1)
        len=No2-No1
        temStr=bytearray(len)
        for i in range(len):    
            temStr[i]=data[No1+i]                       
            
        int_No1= int.from_bytes(temStr,byteorder='little')

        #extract second batch Number
        No1=self.ds.find(self.ds[No2_1],No2)
        No1 +=1
        No2_2=self.ds.find(b'\1f',No1)
        len=No2_2-(No+1)
        for i in range(len):    
            temStr[i]=data[No1+i]                       
                                
        int_No2= int.from_bytes(temStr,byteorder='little')
        No=int_No1+int_No2 
        #adjust pack number
        bNo=int.to_bytes(No,'little ')
        self.ds[No2_1]=bNo

        f_No=self.ds.find(b'\x03',No2_1)   
        s_No=self.ds.find(b'\x03',f_No)
        #l_No=self.ds.find(b'\x03',s_No)

        new_data1=self.ds[0:f_No-6-8]
        new_data2=self.ds[No2_2:]
        new_data3=new_data1+new_data2
    
        return new_data3
    
    def getRecycle(self):
        No1=3
        No2_1=self.ds.find(b'\x1f',No1)
        len=No2_1-No1
        temStr=bytearray(len)
        for i in range(len):    
            temStr[i]=self.ds[No1+i] 

        #extract first batch Number           
        No1=No2_1+1
        No2=self.ds.find(b'\x1f',No1)
        len=No2-No1
        temStr=bytearray(len)
        for i in range(len):    
            temStr[i]=self.ds[No1+i]                       
            
        int_No1= int.from_bytes(temStr,byteorder='little')
        No=int_No1*2
        bNo= int.from_bytes(No,'little ')
        self.ds[No2_1+1]=bNo
        No3=self.ds.find(b'\x03')
        new_data1=self.ds[No2+1:No3-6-8] #x1f50/x1f50.....
        new_data2=self.ds[:No3-6-8] #first string datax x02P/x1fCorn/x1f10/x1f50/.....
        new_data3=self.ds[No3-6-8:] #x1f0/x1f0/x1f5/x1f50/1fStart/
        new_data4=new_data2+new_data1+new_data1+new_data3

        return new_data4 

    def crcCheck_error_respond(self,data):
        data1=[]
        data1.append(data[0])   #get address
        data1.append(data[1])   #get function code
        data1.append(0x04)      #crc check error code
        strData=self.crc16(bytearray(data1),0,len(data1))
        t=divmod(strData,256)
        data1.append(int(t[1]))
        data1.append(int(t[0]))
        self.ser.write(bytes(data1))

    #rs485 respond data
    def rs485_rep_10(self,data):  #for function code 0x0A and 0x06
        strData=[]
        for i in range(6):
            strData.append(data[i])
        #crc16 remainder
        crc16_1=self.crc16(bytearray(strData),0,len(strData))
        x=divmod(crc16_1,256)
        strData.append(int(x[1]))
        strData.append(int(x[0]))
        time.sleep(0.02)
        print(f'send data={strData}')
        self.ser.write(bytes(strData))

    def rs485_rep_04(self,data):
        crc16_1=self.crc16(bytearray(data),0,len(data))
        x=divmod(crc16_1,256)
        data.append(int(x[1]))
        data.append(int(x[0]))
        time.sleep(0.02)
        self.ser.write(data)
    
    def rs485_process_error(self,dFrr,dDrr):
        #invalid function code
        strData=[0x10,dFrr,dDrr]
    
        #crc16 remainder
        crc16_1=self.crc16(bytearray(strData),0,len(strData))
        x=divmod(crc16_1,256)
        strData.append(int(x[1]))
        strData.append(int(x[0]))
        time.sleep(0.02)
        print(f'send data={strData}')
        self.ser.write(bytes(strData))

    '''
    implementation of CRC-16/CCITT-FALSE
        data : bytearray of the data you want to calculate CRC for
        offset : from which offset you want to start calculating CRC
        length : to which offset you want to calculate CRC
    '''
    def crc16(self,data : bytearray, offset, length):
        if data is None or offset < 0 or offset > len(data) - 1 and offset + length > len(data):
            return 0
        print("data length=", len(data))
        print(f'data={data}')

        crc = 0xFFFF
        for i in range(length):
            crc ^= data[offset + i]
            for j in range(8):
                print(crc)
                if ((crc & 0x1) == 1):
                    print("bb1=", crc)
                    crc = int((crc / 2)) ^ 40961
                    print("bb2=", crc)
                else:
                    crc = int(crc / 2)
        return crc & 0xFFFF
    
    def closePort(self):
        self.ser.close()    #close port

if __name__=="__main__":
    try:
        choice = input("Enter 1--rs232.\nEnter 2--rs485rtu\nEnter 3--exit\n")
        choice = int(choice)
        if choice==1:
            serEsc=serialEsc('rs232')
            if serEsc.ser.isOpen():
                iFlag=False
                print(f'ENQ={ENQ}') 
                print(f'ACK={ACK}')
                #check Ping
                while True:
                    if serEsc.Ping() :      
                        print('Get ping !')
                        break
                
                #read data
                while True:
                    data=serEsc.readData()
                    if data:
                        t=data.decode()
                        print(f'Data:{data}')
                        break
                    else:
                        time.sleep(1)
                #send data
                print('I am sending data !\n Hello buddy !')
                serEsc.sendData('Hello,Buddy !\n')
            else:
                print('initial rs232 failed !')
        elif choice==2:
            serEsc=serialEsc('rs485rtu')
            if serEsc.ser.isOpen():
                rs485Set=rs485set()
                rs485Set.read485()
                while True:
                    data=serEsc.readData485()
                    if data[0]==0x10: #check address
                        print('get data at address:0x10 !')
                         #crc16 check
                        rs485Set.send485() #enable rs485 to send data
                        if serEsc.crc16(data,0,len(data))==0x0000:
                            print('Data received, crc check passed!')
                            if data[1]==0x10 :
                                print(f'Received data={data}')
                                serEsc.rs485_rep_10(data)
                                print('respond data={data}')
                                time.sleep(1)
                                rs485Set.read485()
                        else:
                            print('CRC check failed !CRC')
                            serEsc.crcCheck_error_respond(data)
                    else:
                        print('it is not mine. Address:',data[0]) 
            else:
                print('initial rs485rty failed !')   
        else:
            print('exit test !')
            
    except KeyboardInterrupt:
        #close port
        print('Close port')
        serEsc.closePort()
        print('exit test !')
    except:
        #close port
        print('Close port')
        serEsc.closePort()
        print('Rs communication intial failed')
    




   

        
